{{
  Common setup constants and parameters
}}

CON

' Ranges and bit field definitions

  TIF_max       = 8             '# of possible TIF (must be multiple of 4)

' Transaction Info Field Style Bits

  TIF_len_mask  = %0000_0000_0011_1111    'bits used for length of field in bytes

  #6  'start at 2^6 = %0100_0000
  '
  TIF_alpha     'as opposed to numeric
  TIF_uconly    'as opposed to mixed-case alpha
  TIF_entin     'prompt to enter on initiate
  TIF_reqin     'require on innitiate
  TIF_entfin    'prompt to edit on finish
  TIF_reqfin    'require before finish
  TIF_store     'preserve
  TIF_fixed     'do not allow editing

  ' System Setup Option bits

  #0

  KeyID         'Key ID (TIF 0) instead of Loop to look up transaction
  CardID        'enable card ID functions + Card # as alternate ID
  '
  Store         
  StoreTR       
  KeyTR         
  DropTR        
  
  Sen1inlk      
  Sen2inlk      
  SenDir        
  TSdir         
  '
  Cont          'Continuous protocol, only baud used
  SMA           'SMA protocol, only baud used
  '
  Clk12         '12 hour AM-PM clock
  DateDDMM      'DD-MM-YY format date

  ' Instrument setup byte offsets
  ' Room is reserved in setup for 16 of these

  #0
  
  fr_start_ch
  fr_end_ch
  data_start_pos
  data_field_len
  id_ch_pos
  id_ch
  ec1_ch_pos
  ec1_ch
  ec2_ch_pos
  ec2_ch

  ' password indeces

  PW_Len = 8
  '
  #0
  PW_Setup
  PW_Key
  PW_Extra
  PW_Max

  ' Base offsets within setup block

  ' Long aligned byte addresses

  Headers = 0 '4 lines x 32 bytes

  Signature = $80
  
  Sys_Opt = Signature + 4

  Sys_Parms = Sys_Opt + 4
  Sys_RS_Inlk = sys_Parms
  Sys_Instr_Baud = Sys_RS_Inlk + 4
  Sys_Prn_Baud = Sys_Instr_Baud + 4
  Sys_End = Sys_Prn_Baud + 4 

  PWDs = Sys_End
  PW_End = PWDs + PW_Max * 8 
  
  Instr = PW_End

  TIF_styles = Instr + 16

  TIF_names = TIF_styles + TIF_max * 4

  Setup_End = TIF_names + TIF_max * 16

  ' Setup block base

  Setup_Base = $7E00
  Setup_EE_Base = $8000

  EE_SCL = 28  ' EEPROM pins
  EE_SDA = 29

  EE_Signature = 42 

dat

' Text descriptions of setup fields

Sys_Opt_Desc

        byte "Use Keyed ID", 13
        byte "Use Cards", 13
        byte "Store Users", 13
        byte "Store Transactions", 13
        byte "Enable Keyed Data", 13
        byte "Enable Drop Data", 13
        byte "Sensor 1 Blocks", 13
        byte "Sensor 2 Blocks", 13
        byte "Sense Direction", 13
        byte "Apply Direction", 13
        byte "Continuous Format Data", 13
        byte "SMA Format Data", 13
        byte "12 Hour AM/PM Clock", 13
        byte "DD-MM-YYYY date"
        byte 0

Sys_Parm_Desc

        byte "ReSample Interlock",13
        byte "Instrument Baud Rate", 13
        byte "Printer Baud Rate"
        byte 0

TIF_Style_Desc

        byte "Alphanumeric", 13
        byte "Alpha Uppercase", 13
        byte "Enter on Initiate", 13
        byte "Require on Initiate", 13
        byte "Enter on Finish", 13
        byte "Require on Finish", 13
        byte "Store", 13
        byte "Disallow Editing"
        byte 0

Instr_Field_Desc

        byte "Frame Start Char Value", 13
        byte "Frame End Char Value", 13
        byte "Data Start Position", 13
        byte "Data Length", 13
        byte "ID Char Position", 13
        byte "ID Char Value", 13
        byte "Error Char 1 Position", 13
        byte "Error Char 1 Bad Value", 13
        byte "Error Char 2 Position", 13
        byte "Error Char 2 OK Value"
        byte 0

obj

  bsf    : "bufstrfmt"
  vid       : "vga_40x18_rom_text"
        
pub Header(index) : ptr

  ptr := bsf.bufbuf(constant(setup_base + headers) + index << 5, 33)
  byte[ptr+33] := 0 'force null term

pub SetHeader(index, ptr)

  bytemove(constant(setup_base + headers) + index << 5, ptr, 32)  

pub Sys(bit)

  return ((long[constant(setup_base + sys_opt)] >> bit) & 1) <> 0

pub SysDescStr

  return @sys_opt_desc

pub SysParmStr

  return @sys_parm_desc

pub PW(index) : ptr

  ptr := bsf.bufbuf(constant(setup_base + pwds) + index << 3, 9)
  byte[ptr+8] := 0

pub SetPW(index, ptr)

  bytemove(PWDs + index * 8, ptr, 8) 

pub TIFstyle(TIF, bit)

  return ((long[TIF_Styles + TIF << 2] >> bit) & 1) <> 0

pub SetTIFstyle(TIF, bit, val)

  case val
    0:
      long[TIF_Styles + TIF << 2] &= !(1 << bit)
    other:
      long[TIF_Styles + TIF << 2] |= (1 << bit)

pub TIFstyleDescStr

  return @tif_style_desc

pub TIFname(TIF) : ptr

  'Need to supply null terminator if name is 16 char long
  '
  ptr := bsf.bufbuf(constant(setup_base + TIF_names) + TIF << 4, 17)
  byte[ptr+16] := 0

pub SetTIFname(TIF, ptr)

  'This will catch null terminator no matter the length
  '
  bytemove(constant(setup_base + TIF_names) + TIF * 16, ptr, 16)

pub InstrDescStr

  return @instr_field_desc
