{{
  Generic higher-level user interface methods which may involve
  multiple UI objects, intended for multiple inclusion
}}

OBJ

  'Note all objects must be DAT single-instance or local-only
  'multi-instance safe so this object will be too

  vid   : "vga_40x18_rom_text"
  key   : "keyboard"
  sf    : "strfmt"
  bsf   : "bufstrfmt"
  clock : "clock"
' bkg object must have a PUB SERVICE
  bkg   : "bkg"

DAT

' RectMsg descriptors

L       byte  0         'left
T       byte  0         'top
W       byte  0         'width
H       byte  0         'height

PUB DrawHeader(msg, x, y)
{
  CLS, init colors for editing, and draw alt color header
  also refresh the clock which just got overwritten
}
  vid.str(string(1, $C, %%2022))   'set color for CLS
  vid.cls
  clock.display
  vid.setpos(x, y)
  vid.str(string($C, %%2222, $C, %100))
  vid.str(msg)


PUB StRecDims(buf) | LW
{
  This determines the number of screen lines and width in characters
  of a multi-line message at buf, assuming no nonprinting formatting
  characters.  Sets DAT H and W and returns both results in one long
  as H + W<<8
}
  H := 1
  W := 0
  LW := 0
  repeat strsize(buf)
    case byte[buf++]
      13:
        H++
        LW := 0
      other:
        LW++
        if LW > W
          W := LW
  return H + W << 8             'for use by other objects

PUB ScreenMsg(Msg, dcolor) 
{
  Automatically size and center a multi-line message
  and RectMsg it.  Does not save the current screen
  but does performs the DimScreen.
}
  vid.dimscreen(dcolor)
  clock.display 'doesn't get dimmed
  strecdims(msg)
  l := (vid#cols - w - 2) >> 1
  t := (vid#rows - h - 2) >> 1
  vid.rectmsg(l, t, w, h, msg)

PUB FlashMsg(msg, dcolor, sec) | c0, vbuf[379]

  vid.savescreen(@vbuf)
  screenmsg(msg, dcolor)
  c0 := cnt
  repeat until cnt - c0 > sec * clkfreq
    bkg.service
  vid.restorescreen(@vbuf)

pub AbortNotify(msg) | vbuf[379]

  vid.savescreen(@vbuf)
  screenmsg(msg, %%200)
  repeat while waitkey <> $0D
  vid.restorescreen(@vbuf)

pub ConfirmAbandon

  return ConfirmYN(string("Abandon Changes?",13,13,"[Y]es or [N]o..."), %%200)

pub ConfirmYN(msg, dcolor) | c, vbuf[379]

  vid.savescreen(@vbuf)
  screenmsg(msg, dcolor)
  repeat
    c := waitkey
    case c
      "Y", "y":
        vid.restorescreen(@vbuf)
        return -1
      "N", "n", 13:
        vid.restorescreen(@vbuf)
        return 0

pub DrawLetterList(msg, X, Y) | ptr, c, i
{
  Draw the multi-line msg centered if X<0 or at X, Y
  Sets w, h, l, and t for further processing
}
  vid.setcolor(vid.getcolor & vid#colormask)
  strecdims(msg)
  w += 3 'A:_
  l := x
  if x < 0
    l := (vid#cols - w) >> 1
  t := y
  ptr := msg
  i := 0
  vid.setpos(l, t)  
  vid.str(string("A: "))
  repeat strsize(msg)
    case c := byte[ptr++]
      13:
        i++
        vid.setpos(l, t + i)
        vid.out(65 + i)
        vid.out(":")
        vid.out(" ")
      other:
        vid.out(c)
  return H + W << 8
  
pub PickList(msg, X, Y, sel) | c
{
  Letter or up/down to pick an item from a list.
  Y must be specified.  Set X to -1 to center.
  Uses current color non-inverted.
  Sel initializes pick position, -1 preselects nothing.
}
  DrawLetterList(msg, X, Y)
  repeat
    if sel => 0
      hilite(l, t+sel, w, 1)
    c := waitkey
    if sel => 0
      hilite(l, t+sel, w, 0)
    case c
      "a".."z":
        c -= constant("a"-"A")
    case c
      "A".."Z":
        if c - "A" < h
          sel := c - "A"
      $c2:
        'up
        if --sel < 0
          sel := h - 1
      $c3:
        'down
        if ++sel => h
          sel := 0
      $0D:
        if sel => 0
          return sel
      $CB:
        return -1 'ESCAPE

pub EditBits(msg, X, Y, patt) | c, sel, BitX, orgpatt
{
  Pick and toggle the bits of a pattern for setup.
  The number of bits must be < 32.

  On return: -1 user pressed ESCAPE to exit.
}
  DrawLetterList(msg, x, y)
  BitX := l + w + 2
  w += 4
  repeat sel from 0 to h - 1
    vid.setpos(BitX-1, t + sel)
    vid.out("[")
    c := " "
    if (patt & (1 << sel))
      c := "*"
    vid.out(c)
    vid.out("]")
  sel := 0
  patt &= $7FFF_FFFF
  orgpatt := patt 
  hilite(l, t+sel, w, 1)
  repeat
    c := waitkey
    hilite(l, t+sel, w, 0)
    case c
      "a".."z":
        c -= constant("a"-"A")
    case c
      "A".."Z":
        if c - "A" < h
          sel := c - "A"
      $c2:
        'up
        sel--
        if sel < 0
          sel := h - 1
      $c3:
        'down
        sel++
        if sel => h
          sel := 0
      $0D:
        return patt
      32:
        if patt & (1 << sel)
          patt &= !(1 << sel)
          c := " "
        else
          patt |= (1 << sel)
          c := "*"
        vid.setpos(BitX, t + sel)  
        vid.out(c)
      $CB:
        'ESCAPE
        if orgpatt == patt
          return -1
        elseif confirmabandon
          return -1
    hilite(l, t+sel, w, 1)


pub HiLite(X, Y, len, state)

  repeat len
    vid.pokerev(x++, y, state)

pub PopUpInput(msg, buf, len, dcolor) | vbuf[379]
{
  NB the msg must leave room on the right side of the last
  line for the input control.
}
  vid.savescreen(@vbuf)
  screenmsg(msg, dcolor)
  result := doinput(buf, l + w - (len & 63) + 1, t + h, len) 
  vid.restorescreen(@vbuf)

con
  InputNumOnly = 64
  InputHide    = 128
  InputUCase   = 256

pub DoInput(Buf, X, Y, Len) | default[16], c, cursor, NumOnly, AlphaRet, UCase, Hide
{
  Accept operator input while servicing background
  processes.  If there is a default in the Buf it
  is preserved if the user just presses ENTER.
  Editing is in the current color non-reversed.
  Buffer must be Len + 1 to accommodate null term
  Enter and up/down arrows complete entry.
  Add constants to len for options.

  For navigating letter lists this provides for
  exiting a numonly input by typing a letter.
  The highest letter to be accepted for this
  (with A starting as 1) is <<8 and added to Len
  for this feature.

  Numonly input is automatically right-justified
  in the input field on exit.  
}
  numonly := (len & 64) <> 0
  if numonly
    alpharet := (len >> 8) & $FF
    ucase~~
  else
    alpharet~
    ucase := (len & InputUCase) <> 0
  hide := (len & inputhide) <> 0
  len &= 63
  vid.setcolor(vid.getcolor & vid#colormask)
  '
  cursor := 0
  vid.setpos(x, y)
  repeat len
    vid.out(" ")
  vid.setcol(x)
  if byte[buf] <> 0 and strsize(buf) =< len
    vid.str(buf)
    vid.setcol(x)
    repeat
      c := waitkeycursor(x, y)
      if alpharet
        case c
          "a".."z":
            c -= constant("a"-"A")
        if c => "A" and c < "A" + alpharet
          'alpha return within range
          return c
      case c
        13, $C2, $C3, $CB:
          return c
        $C1, $C5:
          'right arrow, end: do nothing
          quit
      if numonly
        case c
          $C8, $C9, "-", "0".."9":
            quit
      else
        case c
          $C8, $C9, " ".."~":
            quit
    'if we fall through we started typing over the default
    case c
      $C1, $C5:
        'right arrow, end: do nothing
      other:
        'blank out entry
        repeat len
          vid.out(" ")
        vid.setcol(x)
        bytemove(@default, buf, len+1)
        byte[buf] := 0
  else
    byte[@default] := 0
    c := waitkeycursor(x, y)
  repeat
    vid.setcol(x+cursor)
    case c
      13, $C2, $C3:
        'ENTER
        if numonly
          vid.setcol(x)
          vid.str(bsf.rdec(sf.val(buf),len))
        return c
      $CB:
        'ESC
        bytemove(buf, @default, len+1)
        vid.setcol(x)
        repeat len
          vid.out(" ")
        vid.setcol(x)
        vid.str(buf)
        return c
      $C8:
        'backspace
        if cursor
          cursor--
          bytemove(buf+cursor, buf+cursor+1, strsize(buf) - cursor)
          vid.out(8)
          vid.str(buf+cursor)
          vid.out(" ")
        c := 0
      $C0:
        'left arrow
        if cursor
          cursor--
      $C1:
        'right arrow
        if byte[buf+cursor]
          cursor++
      $C4:
        'home
        cursor := 0
      $C5:
        'end
        cursor := strsize(buf)
      $CA:
        'insert
        if byte[buf+cursor] and strsize(buf) < len
          bytemove(buf+cursor+1, buf+cursor, strsize(buf)-cursor+1)
          byte[buf+cursor] := " "
          vid.str(buf+cursor)  
      $C9:
        'delete
        if byte[buf+cursor]
          bytemove(buf+cursor, buf+cursor+1, strsize(buf)-cursor)
          vid.str(buf+cursor)
          vid.out(" ")
    if ucase
      case c
        "a".."z":
          c -= constant("a"-"A")
    if numonly
      if c => "A" and c < "A" + alpharet
        'alpha return within range
        vid.setcol(x)
        vid.str(bsf.rdec(sf.val(buf),len))
        return c
      case c
        "-","0".."9":
          'okay
        other:
          c := 0
    if cursor < len
      case c
        " ".."~":
          vid.setcol(x+cursor)
          t := byte[buf+cursor]
          byte[buf+cursor++] := c
          if t == 0
            byte[buf+cursor] := 0
          if hide
            vid.out("*")
          else
            vid.out(c)
    if cursor < len
      c := waitkeycursor(x + cursor, y)
    else
      c := waitkey  

pub MultiNumInput(Desc, Vals, DX, Y, IW) : changes | fields, width, IX, c, buf[3], image[24], sel, val               
{
  Do multi-field numeric input with letter shortcuts and cursor navigation.

  Desc is ptr to a CR-separated null-termanted list of descriptions.
  Vals is ptr to a vector of Long defautl values.
  DX, Y is upper left corner of descriptive block, including added letters.
  Width to highlight and X location of input field block determined from Desc.
  IW is input width.
  Verifies abandoning changes.  Returns True if changes have been made
  and saved, False if no changes have been made
}
  val := DrawLetterList(Desc, DX, Y)
  fields := val & $FF
  width := val >> 8
  vid.setpos(DX, Y + fields)
  vid.out(65+fields)
  vid.str(string(": Save Changes ",$BA,$9C))
  '
  longmove(@image, Vals, fields)
  '
  IX := DX + width + 2
  repeat sel from 0 to fields - 1
    vid.setpos(IX, Y + sel)
    vid.str(bsf.rdec(Image[sel], IW))
  '
  sel~
  repeat
    hilite(DX, Y+sel, width, 1)
    case sel
      fields:
        c := waitkey
      other:
        sf.rdec(@buf, Image[sel], IW) 
        c := doinput(@buf, IX, Y+sel, IW+inputnumonly+(fields+1)<<8)
        if c <> $CB
          Image[sel] := sf.val(@buf)
    hilite(DX, Y+sel, width, 0)
    changes := not sf.bufcomp(vals, @image, fields << 2) 'true if changes made
    case c
      $c2:
        'up
        sel--
        if sel < 0
          sel := fields
      $c3, $0d:
        'down
        sel++
        if sel > fields
          if c == $0d
            'save changes
            longmove(vals, @image, fields)
            return
          sel := 0
      "A".."A"+fields:
        sel := c - "A"
      $cb:
        if not changes
          return
        if confirmabandon
          changes := false
          return
  

pub WaitKeyCursor(X, Y) | cstate, cnew
{
  Wait for a character to be typed while blinking
  the cursor and servicing background processes
}
  cstate := -1
  repeat
    bkg.service
    if (cnew := (cnt >> 25) & 1) <> cstate
      cstate := cnew
      vid.pokerev(x, y, cstate)
    if (result := key.key) <> 0
      vid.pokerev(x, y, 0)
      return

pub WaitKey
{
  Wait for a character to be typed while servicing
  background processes
}
  repeat
    bkg.service
    
    if (result := key.key) <> 0
      return
    